/*
 * Copyright (c) 2011 Tomasz Boleslaw CEDRO <cederom@tlen.pl>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.

 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software Foundation,
 * Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

/** @file Simple framework to bitbang the interface pins.
 */

#include <helper/command.h>
#include <jtag/interface.h>
#include <interface/interface.h>
#include "bitbang.h"

extern struct jtag_interface *jtag_interface;

/*-----------------------------------------------------------------------*/

/*
 * Infrastructure for Tcl interface to bitbang.
 */


COMMAND_HANDLER(handle_bitbang_command)
{
	LOG_DEBUG("%s", __func__);

	if (!jtag_interface){
		LOG_ERROR("You need to setup interface first!");
		return ERROR_FAIL;
	}

	if (!jtag_interface->bitbang){
		LOG_ERROR("This interface does not support bit-banging!");
		return ERROR_FAIL;
	}

	if (CMD_ARGC < 1) {
		LOG_ERROR("Bad syntax!");
		return ERROR_COMMAND_SYNTAX_ERROR;
	}

	if (jtag_interface->signal==NULL){
		LOG_ERROR("No signals defined, see 'interface_signal' command for help.");
		return ERROR_FAIL;
	}

	oocd_interface_signal_t *sig;
	unsigned int pn;
	int retval, *sigval;
	char *mark, *signame;

	// Iterate through list of command parameters
	for (pn=0;pn<CMD_ARGC;pn++){
		// Look for '=' mark to see if read or write will happen
		mark=strnstr(CMD_ARGV[pn], "=", 45);
		if (!mark){
			// If no '=' was found then we read the signal value
			// Check if specified signal exists
			if (!(sig=oocd_interface_signal_find((char *)CMD_ARGV[pn]))){
				LOG_ERROR("Unknown signal specified!");
				return ERROR_FAIL;
			}
			// Call the driver routine to do the actual signal read
			retval=jtag_interface->bitbang(NULL, (char *)CMD_ARGV[pn], 1, sigval);
			if (retval!=ERROR_OK){
				LOG_ERROR("Unable to read signal: %s", CMD_ARGV[pn]);
				return ERROR_FAIL;
			} 
			sig->value=*sigval;
		} else {
			// If '=' was found then we write specified value to the specified signal
			signame=strsep((char **)&CMD_ARGV[pn], "=");
			sig=oocd_interface_signal_find(signame);
			if (!sig){
				LOG_ERROR("Unknown signal specified!");
				return ERROR_FAIL;
			}
			*sigval=sig->value=atoi(CMD_ARGV[pn]);
			retval=jtag_interface->bitbang(NULL, signame, 0, (int *)sig->value);
			if (retval!=ERROR_OK){
				LOG_ERROR("Unable to write signal: %s", signame);
				return ERROR_FAIL;
			}
		}
		command_print(CMD_CTX, "%s=0x%08X", sig->name, sig->value);
	}
	return ERROR_OK;
}

static const struct command_registration bitbang_commands[] = {
	{
		.name = "bitbang",
		.handler = handle_bitbang_command,
		.mode = COMMAND_ANY,
		.help =  "Perform bit-bang operations on interface signals",
		.usage = "[signal_name] to read, [signal_name]=[value] to write.",
	},
	COMMAND_REGISTRATION_DONE
};

int bitbang_register_commands(struct command_context *ctx)
{
	return register_commands(ctx, NULL, bitbang_commands);
}
